CREATE OR REPLACE PACKAGE BODY PKG_DYNAMIC IS

  -------------------------------------------------------------
  --  Sample package to test the 100%dynamic Forms features  --
  --                                                         --
  --  Author : Francois Degrelle                             --
  --                                                         --    
  -------------------------------------------------------------

  ----------------------------------------------------------
  --  Get Datas and data structure from any select Order  --
  ----------------------------------------------------------
  PROCEDURE dynamic_cursor
  (
     PC$Query IN  VARCHAR2,
     PT$Cols  OUT NOCOPY T2,
     PT$rows  OUT NOCOPY T2,
	 PC$Where IN  VARCHAR2 DEFAULT NULL	 
  )
  IS
  c           NUMBER;
  d           NUMBER;
  col_cnt     PLS_INTEGER;
  rec_tab     dbms_sql.desc_tab;
  col_num     NUMBER;

  LN$Lig      PLS_INTEGER := 0 ;
  LN$MaxCol   PLS_INTEGER := 0 ;
  
  LC$Query    VARCHAR2(4000) := PC$Query ;

  v           VARCHAR2(4000) ;
  t           T1 ;

  source_cursor      INTEGER;
  result             INTEGER;

  BEGIN

    -- retrieve the columns of the query --
    c := dbms_sql.open_cursor;

    dbms_sql.parse(c, LC$Query , dbms_sql.NATIVE);

    d := dbms_sql.execute(c);

    dbms_sql.describe_columns(c, col_cnt, rec_tab);


    LN$MaxCol := rec_tab.last ;

    For i in rec_tab.first .. rec_tab.last Loop

      PT$Cols(i)(1) := rec_tab(i).col_name ;      -- name
      PT$Cols(i)(2) := rec_tab(i).col_type ;      -- type
      PT$Cols(i)(3) := rec_tab(i).col_precision ; -- precision
      PT$Cols(i)(4) := rec_tab(i).col_scale ;     -- scale
      PT$Cols(i)(5) := rec_tab(i).col_max_len ;   -- length

    End loop ;

    dbms_sql.close_cursor(c);


    -- Prepare a cursor to select from the source table: --
    source_cursor := dbms_sql.open_cursor;
	
    -- Where clause ? --
	If PC$Where IS NOT NULL Then
	   LC$Query := PC$Query || PC$Where ;
	End if ;

    dbms_sql.parse(source_cursor,  LC$Query, dbms_sql.NATIVE);


    -- Define the columns --
    For i in 1 .. LN$MaxCol Loop
      dbms_sql.DEFINE_COLUMN(source_cursor, i, v,4000);
    End loop ;


    result := dbms_sql.execute(source_cursor);


    -- Fetch the rows from the source query  --
    LOOP
      IF dbms_sql.FETCH_ROWS(source_cursor)>0 THEN
        -- get column values of the row --
        LN$Lig := LN$Lig + 1 ;
        For i in 1.. LN$MaxCol Loop
          dbms_sql.COLUMN_VALUE(source_cursor, i, t(i));
          PT$rows(LN$Lig)(i) := t(i) ;
        End loop ;

      ELSE
        -- No more rows --
        EXIT;
      END IF;
    END LOOP;


    dbms_sql.close_cursor(source_cursor);


  EXCEPTION
    WHEN OTHERS THEN

      IF dbms_sql.is_open(source_cursor) THEN
         dbms_sql.close_cursor(source_cursor);
      END IF;

      RAISE;

  END;

  
  -----------------------------------------------
  --  Get data structure for any select order  --
  -----------------------------------------------
  PROCEDURE Get_Struct
  (
     PC$Query IN  VARCHAR2,
     PT$Cols  OUT NOCOPY T2
  )
  IS
  c           NUMBER;
  d           NUMBER;
  col_cnt     PLS_INTEGER;
  rec_tab     dbms_sql.desc_tab;
  col_num     NUMBER;

  LN$Lig      PLS_INTEGER := 0 ;
  LN$MaxCol   PLS_INTEGER := 0 ;

  v           VARCHAR2(4000) ;
  t           T1 ;

  result             INTEGER;

  BEGIN

    -- retrieve the columns of the query --
    c := dbms_sql.open_cursor;

    dbms_sql.parse(c, PC$Query , dbms_sql.NATIVE);

    d := dbms_sql.execute(c);

    dbms_sql.describe_columns(c, col_cnt, rec_tab);


    LN$MaxCol := rec_tab.last ;

    For i in rec_tab.first .. rec_tab.last Loop

      PT$Cols(i)(1) := rec_tab(i).col_name ;      -- name
      PT$Cols(i)(2) := rec_tab(i).col_type ;      -- type
      PT$Cols(i)(3) := rec_tab(i).col_precision ; -- precision
      PT$Cols(i)(4) := rec_tab(i).col_scale ;     -- scale
      PT$Cols(i)(5) := rec_tab(i).col_max_len ;   -- length

    End loop ;

    dbms_sql.close_cursor(c);


  EXCEPTION
    WHEN OTHERS THEN

      IF dbms_sql.is_open(c) THEN
         dbms_sql.close_cursor(c);
      END IF;

      RAISE;

  END Get_Struct;


  ---------------------------------------
  --  Item value validation function  ---
  ---------------------------------------
  FUNCTION Validation
  ( 
     PN$DlgNum  IN  PLS_INTEGER, 
	 PC$ColName IN  VARCHAR2,
	 PC$Type    IN  VARCHAR2, 
	 PC$Value   IN  VARCHAR2,
	 PC$ErrMsg  OUT VARCHAR2 
  )
  RETURN BOOLEAN
  IS
     Cursor C_VALID IS
	 Select V_RULE, V_ERR_MSG
	 From   DYN_VALIDATION
	 Where  D_NUM = PN$DlgNum
	 And    V_COL = Upper( PC$ColName )
	 ;
	 LR$Rec  C_VALID%ROWTYPE ;
	 LC$Req  Varchar2(1000) ;
	 LN$Ret  Pls_Integer := 0 ;
	 LC$Tab  Varchar2(30) ;
	 LC$Col  Varchar2(30) ;
	 LN$Deb  Pls_Integer ;
	 LN$Fin  Pls_Integer ;	 
  Begin
  
    Open  C_VALID ;
	Loop
	   
	   Fetch C_VALID Into LR$Rec ;
	   Exit when C_VALID%NOTFOUND ;
	   
	   If Instr( LR$Rec.V_Rule, 'EXISTS' ) > 0 Then
	     -- Referential integrity validation --
		 LN$Deb := Instr( LR$Rec.V_Rule, 'TABLE=' ) ;
		 If LN$Deb = 0 Then
		    PC$ErrMsg := 'Table name not found for EXISTS validation' ;
			Return FALSE ;
		 End if ; 
		 LC$Tab := Substr( LR$Rec.V_Rule, LN$Deb + 6, 500 ) ; 
		 LN$Fin := Instr( LC$Tab, ' ' ) ;
		 LC$Tab := Substr( LC$Tab, 1, LN$Fin - 1 ) ;

	     LN$Deb := Instr( LR$Rec.V_Rule, 'COLUMN=' ) ;
		 If LN$Deb = 0 Then
		    PC$ErrMsg := 'Column name not found for EXISTS validation' ;
			Return FALSE ;
		 End if ; 
		 LC$Col := Trim( Substr( LR$Rec.V_Rule, LN$Deb + 7, 500 ) ) ; 

         LC$Req := 'SELECT 1 FROM DUAL WHERE EXISTS( SELECT 1 FROM ' || LC$Tab || ' WHERE ' || LC$Col || ' = ' || PC$Value || ' )';		 
	   Else
         -- formula validation --
	     If PC$Type = 'D' Then
	       LC$Req := 'SELECT 1 FROM DUAL WHERE TO_DATE(''' || PC$Value || ''') ' || LR$Rec.V_Rule ;
	     Else
	       LC$Req := 'SELECT 1 FROM DUAL WHERE ' || PC$Value || ' ' || LR$Rec.V_Rule ;
	     End if ;
	   End if ;
dbms_output.put_line( LC$Req ) ;

	   Begin
	     Execute Immediate LC$Req Into LN$Ret ;
	   Exception
	     When no_data_found Then
		    LN$Ret := 0 ;
	   End ;

	   If LN$Ret = 0 Then
	      
		  PC$ErrMsg := LR$Rec.V_Err_Msg ;
		  
		  Return FALSE ;
		  
	   End if ;
	   
	   
	End loop ;
	
    Return TRUE ;
	
  End Validation ;


  -----------------
  --  Insert row --
  -----------------
  FUNCTION Insert_row
  (
     PN$DlgNum  IN PLS_INTEGER,
	 PT$Struct  IN T2,
	 PT$row     IN T1,
	 PC$Msg     OUT VARCHAR2	 
  )
  RETURN BOOLEAN
  IS
    LC$Req   Varchar2(4000) ;
	LC$Table  Varchar2(30) ;
	Cursor C_T ( PN$Num IN PLS_INTEGER ) IS
	Select D_TABLE_NAME From DYN_DIALOG
	Where  D_NUM = PN$Num ;
  BEGIN
    
    -- Select the Table name --
	Open  C_T( PN$DlgNum ) ;
	Fetch C_T Into LC$Table ;
	Close C_T ;
	If LC$Table IS NULL Then
	   Return FALSE ;
	End if ;
	
	-- Build the insert order --
	LC$Req := 'INSERT INTO ' || LC$Table || ' (' ;

    -- Column description --
	For i IN PT$Struct.First .. PT$Struct.Last Loop
	  If i > 1 Then -- Skip the rowid
	    If i > 2 Then
		  LC$Req := LC$Req || ',' ;
		End if ;
		LC$Req := LC$Req || PT$Struct(i)(1) ;
	  End if ;
	End loop ;
	LC$Req := LC$Req || ') VALUES(' ;
    -- Values --
	For i IN PT$row.First .. PT$row.Last Loop
	  If i > 1 Then -- Skip the rowid
	    If i > 2 Then
		  LC$Req := LC$Req || ',' ;
		End if ;
		
		If PT$Struct(i)(2) = '1' Then
		  LC$Req := LC$Req || '''' || PT$row(i) || '''' ;		
		ElsIf PT$Struct(i)(2) = '12' Then
		  If PT$row(i) IS NULL Then
		     LC$Req := LC$Req || 'NULL' ;
		  Else
		     LC$Req := LC$Req || ' TO_DATE(''' || PT$row(i) || ''')' ;		  
		  End if ;		
		Else		
		  LC$Req := LC$Req || Nvl( PT$row(i), 'NULL' ) ;
		End if ;		
		
		
	  End if ;
	End loop ;
	
	LC$Req := LC$Req || ')' ;
	
    --F_TRACE('Insert : ' || LC$Req ,'T') ;
	
	Execute Immediate LC$Req ;	
	
	Return TRUE ;	
	
  Exception
     When Others Then
		PC$Msg  := SQLERRM ;
		Return FALSE ;	
  END Insert_row ;
  
    
  -----------------
  --  Update row --
  -----------------
  FUNCTION Update_row
  (
     PN$DlgNum  IN PLS_INTEGER,
	 PT$Struct  IN T2,	 
	 PT$row     IN T1,
	 PC$Msg     OUT VARCHAR2	 
  )
  RETURN BOOLEAN
  IS
	LC$Table  Varchar2(30) ;
    LC$Req   Varchar2(4000) ;	
	Cursor C_T ( PN$Num IN PLS_INTEGER ) IS
	Select D_TABLE_NAME From DYN_DIALOG
	Where  D_NUM = PN$Num ;  
  BEGIN

    -- Select the Table name --
	Open  C_T( PN$DlgNum ) ;
	Fetch C_T Into LC$Table ;
	Close C_T ;
	If LC$Table IS NULL Then
	   Return FALSE ;
	End if ;
	
	-- Build the update order --
	LC$Req := 'UPDATE ' || LC$Table || ' SET ' ;

    -- Values --
	For i IN PT$row.First .. PT$row.Last Loop
	  If i > 1 Then -- Skip the rowid
	    If i > 2 Then
		  LC$Req := LC$Req || ',' ;
		End if ;
		If PT$Struct(i)(2) = '1' Then
		  LC$Req := LC$Req || PT$Struct(i)(1) || '=''' || PT$row(i) || '''' ;		
		ElsIf PT$Struct(i)(2) = '12' Then
		  If PT$row(i) IS NULL Then
		     LC$Req := LC$Req || PT$Struct(i)(1) || '= NULL' ;
		  Else
		     LC$Req := LC$Req || PT$Struct(i)(1) || '= TO_DATE(''' || PT$row(i) || ''')' ;		  
		  End if ;		
		Else		
		  LC$Req := LC$Req || PT$Struct(i)(1) || '=' || Nvl( PT$row(i), 'NULL' ) ;
		End if ;
	  End if ;
	End loop ;
	
	LC$Req := LC$Req || ' WHERE ROWID = ''' || PT$row(1) || '''' ;
	
	--F_TRACE('Update : ' || LC$Req , 'T') ;
	
	Execute Immediate LC$Req ;
	
	Return TRUE ;		
	
  Exception
     When Others Then
		PC$Msg  := SQLERRM ;
		Return FALSE ;
  END Update_row ;
  
  
  -----------------
  --  Delete row --
  -----------------
  FUNCTION Delete_row
  (
     PN$DlgNum  IN PLS_INTEGER,
	 PT$row     IN T1,
	 PC$Msg     OUT VARCHAR2	 
  )
  RETURN BOOLEAN
  IS
	LC$Table  Varchar2(30) ;
    LC$Req   Varchar2(4000) ;	
	Cursor C_T ( PN$Num IN PLS_INTEGER ) IS
	Select D_TABLE_NAME From DYN_DIALOG
	Where  D_NUM = PN$Num ;  
  BEGIN

    -- Select the Table name --
	Open  C_T( PN$DlgNum ) ;
	Fetch C_T Into LC$Table ;
	Close C_T ;
	If LC$Table IS NULL Then
	   Return FALSE ;
	End if ;
	
	-- Build the Delete order --
	LC$Req := 'DELETE FROM ' || LC$Table || ' WHERE ROWID = ''' || PT$row(1) || '''' ;
	
	--F_TRACE('Delete : ' || LC$Req , 'T') ;
	
	Execute Immediate LC$Req ;
	
	Return TRUE ;		
	
  Exception
     When Others Then
		PC$Msg  := SQLERRM ;
		Return FALSE ;
  END Delete_row ;


  ------------------------
  --  Get columns info  --
  ------------------------
  Procedure Get_Cols_Info
  (
     PC$DlgName IN  VARCHAR2,
	 PT$Cols    OUT NOCOPY TYP_COLS
  )
  IS
  Begin
     
	  Select *
	  Bulk   collect
	  Into   PT$Cols
	  From   DYN_COLS
	  Where  D_NUM = PC$DlgName ;
	  
  Exception
     When NO_DATA_FOUND Then
	     Null ;
  End Get_Cols_Info ;

  ----------------------------------
  --  Get particular column info  --
  ----------------------------------
  Procedure Get_Col_Info
  (
     PC$DlgName IN  VARCHAR2,
	 PC$ColName IN  VARCHAR2,
	 PR$Col     OUT NOCOPY DYN_COLS%ROWTYPE
  )
  IS
  Begin
     
	  Select *
	  Into   PR$Col
	  From   DYN_COLS
	  Where  D_NUM = PC$DlgName
	  And    C_COL = PC$ColName
	  And    ROWNUM <= 1 ;
	  
  Exception
     When NO_DATA_FOUND Then
	     Null ;
  End Get_Col_Info ;  


  -----------------------------
  --  Lock a particular row  --
  -----------------------------
  FUNCTION Lock_row
  (
     PN$DlgNum  IN  PLS_INTEGER,
	 PT$row     IN  T1,
	 PC$Msg     OUT VARCHAR2	 
  )
  RETURN BOOLEAN
  IS
    LC$TableName  Varchar2(30) ;
	LC$Req        Varchar2(256) ;
  Begin
  
     Select  D_TABLE_NAME
	 Into    LC$TableName
	 From    DYN_DIALOG
	 Where   D_NUM = PN$DlgNum;
	 
	 LC$Req := 'SELECT * FROM ' || LC$TableName || ' WHERE ROWID = ''' || PT$row(1) || ''' FOR UPDATE NOWAIT' ;
	 --f_trace( 'lock : ' || lc$req, 'T' ) ;
	 Execute Immediate LC$Req ;
	 
	 Return TRUE ;
	 

  Exception
     When NO_DATA_FOUND Then
	    PC$Msg := 'Table name for Dialog : ' || To_Char( PN$DlgNum ) || ' not found' ;
		Return FALSE ;
  End Lock_row ;
  

END PKG_DYNAMIC ;
/
