CREATE OR REPLACE PACKAGE PKG_DYNAMIC IS

  -------------------------------------------------------------
  --  Sample package to test the 100%dynamic Forms features  --
  --                                                         --
  --  Author : Francois Degrelle                             --
  --                                                         --    
  -------------------------------------------------------------

  TYPE T1 IS TABLE OF VARCHAR2(4000) INDEX BY BINARY_INTEGER ;
  TYPE T2 IS TABLE OF T1 INDEX BY BINARY_INTEGER ;
  TYPE TYP_COLS  IS TABLE OF DYN_COLS%ROWTYPE INDEX BY PLS_INTEGER ;

  -- Get datas and data structure for any select order --
  PROCEDURE dynamic_cursor
  (
     PC$Query IN  VARCHAR2,
     PT$Cols  OUT NOCOPY T2,
     PT$rows  OUT NOCOPY T2,
	 PC$Where IN  VARCHAR2 DEFAULT NULL
  ) ;
  
  -- Get data structure for any select order --
  PROCEDURE Get_Struct
  (
     PC$Query IN  VARCHAR2,
     PT$Cols  OUT NOCOPY T2
  ) ;  
  
  -- Validation --
  FUNCTION Validation
  ( 
     PN$DlgNum  IN  PLS_INTEGER, 
	 PC$ColName IN  VARCHAR2,
	 PC$Type    IN  VARCHAR2, 
	 PC$Value   IN  VARCHAR2,
	 PC$ErrMsg  OUT VARCHAR2 
  )
  RETURN BOOLEAN ;
  
  -- Insert --
  FUNCTION Insert_row
  (
     PN$DlgNum  IN  PLS_INTEGER,
	 PT$Struct  IN  T2,
	 PT$row     IN  T1,
	 PC$Msg     OUT VARCHAR2
  )
  RETURN BOOLEAN ;
    
  -- Update --
  FUNCTION Update_row
  (
     PN$DlgNum  IN  PLS_INTEGER,
	 PT$Struct  IN  T2,	 
	 PT$row     IN  T1,
	 PC$Msg     OUT VARCHAR2	 
  )
  RETURN BOOLEAN ;
  
  -- Delete --
  FUNCTION Delete_row
  (
     PN$DlgNum  IN  PLS_INTEGER,
	 PT$row     IN  T1,
	 PC$Msg     OUT VARCHAR2	 
  )
  RETURN BOOLEAN ;
  
  -- Get columns info --
  Procedure Get_Cols_Info
  (
     PC$DlgName IN  VARCHAR2,
	 PT$Cols    OUT NOCOPY TYP_COLS
  ) ;
  
  -- Get particular column info --
  Procedure Get_Col_Info
  (
     PC$DlgName IN  VARCHAR2,
	 PC$ColName IN  VARCHAR2,
	 PR$Col     OUT NOCOPY DYN_COLS%ROWTYPE
  ) ;
  
  --  Lock a particular row --
  FUNCTION Lock_row
  (
     PN$DlgNum  IN  PLS_INTEGER,
	 PT$row     IN  T1,
	 PC$Msg     OUT VARCHAR2	 
  )
  RETURN BOOLEAN ;    

  
END PKG_DYNAMIC ;
/
