CREATE OR REPLACE PACKAGE BODY Pkg_Dynamic_2 IS

  -----------------------------------------------------------
  --  Sample package to test the 100%dynamic SQL features  --
  --                                                       --
  --  Author : Francois Degrelle                           --
  --                                                       --    
  -----------------------------------------------------------

  ----------------------------------
  --  Generate the table content  --
  ----------------------------------
  PROCEDURE Generate_Table
  (
     PC$Query IN  VARCHAR2
  )
  IS
  c           NUMBER;
  d           NUMBER;
  col_cnt     PLS_INTEGER;
  rec_tab     dbms_sql.desc_tab;
  col_num     NUMBER;
  pos         PLS_INTEGER ;

  LN$Lig      PLS_INTEGER := 0 ;
  LN$MaxCol   PLS_INTEGER := 0 ;

  v           VARCHAR2(4000) ;
  t           T1 ;

  source_cursor      INTEGER;
  result             INTEGER;

  BEGIN

    DBMS_OUTPUT.ENABLE( 1000000 );
	
	T_Cols.Delete ;
	T_rows.Delete ;
	GC$TName := '' ;
	
	-- retrieve the columns of the query --
    c := dbms_sql.open_cursor;

    dbms_sql.parse(c, PC$Query , dbms_sql.NATIVE);

    d := dbms_sql.EXECUTE(c);

    dbms_sql.describe_columns(c, col_cnt, rec_tab);


    LN$MaxCol := rec_tab.last ;

    FOR i IN rec_tab.first .. rec_tab.last LOOP

      T_Cols(i)(1) := rec_tab(i).col_name ;      -- name
      T_Cols(i)(2) := rec_tab(i).col_type ;      -- type
      T_Cols(i)(3) := rec_tab(i).col_precision ; -- precision
      T_Cols(i)(4) := rec_tab(i).col_scale ;     -- scale
      T_Cols(i)(5) := rec_tab(i).col_max_len ;   -- length

    END LOOP ;

    dbms_sql.close_cursor(c);
	
	-- Retrieve the table name --
    Pos := Instr( Upper( PC$Query ), 'FROM' ) ;
	GC$TName := Substr( PC$Query, Pos + 5 ) ;
	Pos := Instr( GC$TName, ' ' ) ;
	GC$TName := Substr( GC$TName, 1 , Pos -1 ) ;
	
    -- Prepare a cursor to select from the source table: --
    source_cursor := dbms_sql.open_cursor;

    dbms_sql.parse(source_cursor,  PC$Query, dbms_sql.NATIVE);


    -- Define the columns --
    FOR i IN 1 .. LN$MaxCol LOOP
      dbms_sql.DEFINE_COLUMN(source_cursor, i, v,4000);
    END LOOP ;


    result := dbms_sql.EXECUTE(source_cursor);


    -- Fetch the rows from the source query  --
    LOOP
      IF dbms_sql.FETCH_ROWS(source_cursor)>0 THEN
        -- get column values of the row --
        LN$Lig := LN$Lig + 1 ;
        FOR i IN 1.. LN$MaxCol LOOP

          dbms_sql.COLUMN_VALUE(source_cursor, i, t(i));
          T_rows(LN$Lig)(i) := t(i) ;

        END LOOP ;

      ELSE
        -- No more rows --
        EXIT;
      END IF;
    END LOOP;


    dbms_sql.close_cursor(source_cursor);


  EXCEPTION
    WHEN OTHERS THEN

      IF dbms_sql.is_open(source_cursor) THEN
         dbms_sql.close_cursor(source_cursor);
      END IF;

      RAISE;

  END Generate_Table ;


  ---------------------------------
  --  Display the content table  --
  ---------------------------------
  PROCEDURE Show_Table
  IS
    LN$SIZE PLS_INTEGER ;
    LN$Tot  PLS_INTEGER := 0 ;
	LC$Lig  Varchar2(4000) ;  
  Begin

    DBMS_OUTPUT.ENABLE( 1000000 );
	
    FOR i IN T_Cols.First .. T_Cols.Last LOOP
	   IF T_Cols(i)(2) = '1' THEN LN$SIZE := T_Cols(i)(5); END IF ;
	   IF T_Cols(i)(2) = '2' THEN LN$SIZE := GREATEST(T_Cols(i)(3),15); END IF ;
	   IF T_Cols(i)(2) = '12' THEN LN$SIZE := 15; END IF ;
	   LN$Tot := LN$Tot + LN$SIZE + 1 ;
	   LC$Lig := LC$Lig || ( RPAD( SUBSTR( T_Cols(i)(1), 1, LN$SIZE ), LN$SIZE + 1 )) ;
	END LOOP ;
    OutPut( LC$Lig ) ;
	OutPut (' ');
	OutPut ( RPAD( '-', LN$Tot, '-' ) );
	LC$Lig := '' ;
	For i IN T_rows.First .. T_rows.Last Loop
      FOR j IN T_rows(i).First .. T_rows(i).Last LOOP
	     IF T_Cols(j)(2) = '1' THEN LN$SIZE := T_Cols(j)(5); END IF ;
	     IF T_Cols(j)(2) = '2' THEN LN$SIZE := GREATEST(T_Cols(j)(3),15); END IF ;
	     IF T_Cols(j)(2) = '12' THEN LN$SIZE := 15; END IF ;
	     LC$Lig := LC$Lig || ( RPAD( SUBSTR( Nvl( T_rows(i)(j), ' ' ), 1, LN$SIZE ), LN$SIZE + 1 )) ;
	  END LOOP ;
      OutPut( LC$Lig ) ;
	  OutPut (' ');
	  LC$Lig := '' ;
	End loop ;
    OutPut ( RPAD( '-', LN$Tot, '-' ) ) ;

  End Show_Table ;


  ---------------------------------------------
  --  Retrieve and display a particular row  --
  ---------------------------------------------
  PROCEDURE Get_row
  (
     PN$row   IN  PLS_INTEGER,
	 PR$row   OUT T1
  )
  IS
    LN$SIZE PLS_INTEGER ;
	LN$Tot  PLS_INTEGER := 0 ;
	LC$Lig  Varchar2(4000) ;
  BEGIN

    If PN$row < T_rows.First Or PN$row > T_rows.Last Then
	   Dbms_Output.put_Line('********************  ERROR  ********************' ) ;
	   Dbms_Output.put_Line('*    row must be between ' || Ltrim(To_Char(T_rows.First)) || ' and ' || Ltrim(To_Char(T_rows.Last))  ) ;
	   Dbms_Output.put_Line('*************************************************' ) ;	   
	   return ;
	End if ;
	
    FOR i IN T_Cols.First .. T_Cols.Last LOOP
	   IF T_Cols(i)(2) = '1' THEN LN$SIZE := T_Cols(i)(5); END IF ;
	   IF T_Cols(i)(2) = '2' THEN LN$SIZE := GREATEST(T_Cols(i)(3),15); END IF ;
	   IF T_Cols(i)(2) = '12' THEN LN$SIZE := 15; END IF ;
	   LN$Tot := LN$Tot + LN$SIZE + 1 ;
	   LC$Lig := LC$Lig || ( RPAD( SUBSTR( T_Cols(i)(1), 1, LN$SIZE ), LN$SIZE + 1 )) ;
	END LOOP ;
	OutPut( LC$Lig ) ;
    OutPut (' ');
	OutPut ( RPAD( '-', LN$Tot, '-' ) );
	LC$Lig := '' ;
    FOR i IN T_rows(PN$row).First .. T_rows(PN$row).Last LOOP

	   IF T_Cols(i)(2) = '1' THEN LN$SIZE := T_Cols(i)(5); END IF ;
	   IF T_Cols(i)(2) = '2' THEN LN$SIZE := GREATEST(T_Cols(i)(3),15); END IF ;
	   IF T_Cols(i)(2) = '12' THEN LN$SIZE := 15; END IF ;
	   LC$Lig := LC$Lig || ( RPAD( SUBSTR( Nvl( T_rows(PN$row)(i), ' ') , 1, LN$SIZE ), LN$SIZE + 1 )) ;
	END LOOP ;
	OutPut( LC$Lig ) ;
    OutPut (' ');
    OutPut ( RPAD( '-', LN$Tot, '-' ) ) ;
  END Get_row;

  -------------------------------
  --  Delete a particular row  --
  -------------------------------
  FUNCTION Delete_row
  (
     PN$row   IN  PLS_INTEGER
  ) RETURN Boolean
  Is
    LC$Req  Varchar2(256) ;
	PC$Msg  Varchar2(256) ;
  Begin
  
    If PN$row < T_rows.First Or PN$row > T_rows.Last Then
	   Dbms_Output.put_Line('********************  ERROR  ********************' ) ;
	   Dbms_Output.put_Line('*    row must be between ' || Ltrim(To_Char(T_rows.First)) || ' and ' || Ltrim(To_Char(T_rows.Last))  ) ;
	   Dbms_Output.put_Line('*************************************************' ) ;	   
	   Return FALSE ;
	End if ;
	  
    LC$Req := 'DELETE FROM ' || GC$TName || ' WHERE ROWID = ''' || T_rows(PN$row)(T_rows(PN$row).Last) || '''' ;
	Execute Immediate LC$Req ;
	
	Return TRUE ;
	
  Exception
     When Others Then
	    PC$Msg  := SQLERRM ;
		Dbms_Output.put_Line( PC$Msg ) ;
	    Return FALSE ;
  End Delete_row ;



  ------------------------
  --  Insert a new row  --
  ------------------------
  FUNCTION Insert_row
  (
     PC$Values   IN  VARCHAR2
  ) RETURN Boolean
  Is
    LC$Req  Varchar2(2000) ;
	k       Pls_Integer := 0 ;
	PC$Msg  Varchar2(256) ;
  Begin
  
    -- Build insert order --
	LC$Req := 'INSERT INTO ' || GC$TName || '(' ;
    FOR i IN T_Cols.First .. T_Cols.Last LOOP
	   If T_Cols(i)(1) Not IN ('ROWNUM','ROWID' ) Then
	      k := k + 1 ;
		  If k > 1 Then LC$Req := LC$Req || ',' ; End if ;
		  LC$Req := LC$Req || T_Cols(i)(1) ;
	   End if ; 
	END LOOP ;
	LC$Req := LC$Req || ') VALUES(' || PC$Values || ')' ;
	
	Execute Immediate LC$Req ;	
	
	Return TRUE ;
  
  Exception
     When Others Then
	    PC$Msg  := SQLERRM ;
		Dbms_Output.put_Line( PC$Msg ) ;
		Return FALSE ;
  End Insert_row ;    


  -------------------------------
  --  Update a particular row  --
  -------------------------------
  FUNCTION Update_row
  (
     PN$row      IN PLS_INTEGER,
	 PC$Values   IN  VARCHAR2
  ) RETURN Boolean
  Is
     LC$Req   Varchar2(512) ;
	 PC$Msg   Varchar2(256) ;
  Begin
  
    LC$Req := 'UPDATE ' || GC$TName || ' SET ' || PC$Values || ' WHERE ROWID = ''' || T_rows(PN$row)(T_rows(PN$row).Last) || '''' ;
	Execute Immediate LC$Req ;
	
    Return TRUE ;

  Exception
     When Others Then
	    PC$Msg  := SQLERRM ;
		Dbms_Output.put_Line( PC$Msg ) ;
		Return FALSE ;
  End Update_row ;
  


  ----------------------------
  --  Output on the screen  --
  ----------------------------
  PROCEDURE OutPut
  (
    PC$String IN VARCHAR2
  )
  IS
    LC$String Varchar2(4000) ;
    LN$Slices	PLS_INTEGER ;
    LN$Rest  	PLS_INTEGER ;
    LN$Pos	PLS_INTEGER := 1 ;
    LN$Inc	PLS_INTEGER ;  
  Begin
  
	 LN$Inc := 255 ;
	 LN$Slices := Length( PC$String ) / LN$Inc ;
	 LN$Rest    := MOD( Length( PC$String ), LN$Inc ) ;
	 If LN$Rest > 0 Then LN$Slices := LN$Slices + 1 ; End if ;

	 -- output --
	 For i in 1..LN$Slices Loop
	    LC$String := Substr( PC$String, LN$Pos, LN$Inc ) ;
		DBMS_OUTPUT.PUT_LINE( LC$String ) ;
		LN$Pos := LN$Pos + LN$Inc ;
	 End loop ;  
  
  End OutPut ;
  
  		
  
END Pkg_Dynamic_2 ;
/
