
DROP TABLE DEPT CASCADE CONSTRAINTS
/

CREATE TABLE DEPT
(
  DEPTNO  NUMBER(2),
  DNAME   VARCHAR2(14 BYTE),
  LOC     VARCHAR2(13 BYTE)
)
/


DROP TABLE EMP CASCADE CONSTRAINTS
/

CREATE TABLE EMP
(
  EMPNO     NUMBER(4)                           NOT NULL,
  ENAME     VARCHAR2(10 BYTE),
  JOB       VARCHAR2(9 BYTE),
  MGR       NUMBER(4),
  HIREDATE  DATE,
  SAL       NUMBER(7,2),
  COMM      NUMBER(7,2),
  DEPTNO    NUMBER(2)
)
/

DROP TABLE DYN_DIALOG CASCADE CONSTRAINTS
/

CREATE TABLE DYN_DIALOG
(
  D_NUM         NUMBER(5),
  D_NAME        VARCHAR2(100 BYTE),
  D_TITLE       VARCHAR2(256 BYTE),
  D_QUERY       VARCHAR2(2000 BYTE),
  D_GRANT       VARCHAR2(50 BYTE),
  D_TABLE_NAME  VARCHAR2(30 BYTE)
)
/


CREATE UNIQUE INDEX DYN_DLG_PK ON DYN_DIALOG
(D_NUM)
LOGGING
NOPARALLEL
/


ALTER TABLE DYN_DIALOG ADD (
  CONSTRAINT DYN_DLG_PK PRIMARY KEY (D_NUM))
/

DROP TABLE DYN_COLS CASCADE CONSTRAINTS
/

CREATE TABLE DYN_COLS
(
  D_NUM        NUMBER(5),
  C_COL        VARCHAR2(30 BYTE),
  C_LABEL      VARCHAR2(100 BYTE)               NOT NULL,
  C_INITVALUE  VARCHAR2(100 BYTE),
  C_REQUIRED   VARCHAR2(1 BYTE)                 DEFAULT 'N',
  C_QUERYABLE  VARCHAR2(1 BYTE)                 DEFAULT 'Y',
  C_CASE       VARCHAR2(1 BYTE)                 DEFAULT 'M',
  C_MASK       VARCHAR2(50 BYTE),
  C_VISIBLE    VARCHAR2(1 BYTE)                 DEFAULT 'Y'
)
/

CREATE UNIQUE INDEX PK_DYN_COLS ON DYN_COLS
(D_NUM, C_COL)
LOGGING
NOPARALLEL
/

ALTER TABLE DYN_COLS ADD (
  CHECK (C_REQUIRED IN ('Y','N')))
/

ALTER TABLE DYN_COLS ADD (
  CHECK (C_QUERYABLE IN('Y','N')))
/

ALTER TABLE DYN_COLS ADD (
  CHECK (C_CASE IN('U','L','M')))
/

ALTER TABLE DYN_COLS ADD (
  CHECK (C_VISIBLE IN('Y','N')))
/

ALTER TABLE DYN_COLS ADD (
  CONSTRAINT PK_DYN_COLS PRIMARY KEY (D_NUM, C_COL))
/

DROP TABLE DYN_VALIDATION CASCADE CONSTRAINTS
/

CREATE TABLE DYN_VALIDATION
(
  D_NUM      NUMBER(5),
  V_COL      VARCHAR2(60 BYTE),
  V_NUM      NUMBER(2),
  V_RULE     VARCHAR2(100 BYTE),
  V_ERR_MSG  VARCHAR2(256 BYTE)
)
/

CREATE UNIQUE INDEX DYN_VAL_PK ON DYN_VALIDATION
(D_NUM, V_COL, V_NUM)
LOGGING
NOPARALLEL
/

ALTER TABLE DYN_VALIDATION ADD (
  CONSTRAINT DYN_VAL_PK PRIMARY KEY (D_NUM, V_COL, V_NUM))
/

INSERT INTO DYN_DIALOG ( D_NUM, D_NAME, D_TITLE, D_QUERY, D_GRANT,
D_TABLE_NAME ) VALUES ( 
1, 'Employees', 'Play with the Employees table', 'SELECT t.ROWID, t.* FROM EMP t'
, 'I,U,D', 'EMP'); 
INSERT INTO DYN_DIALOG ( D_NUM, D_NAME, D_TITLE, D_QUERY, D_GRANT,
D_TABLE_NAME ) VALUES ( 
2, 'Departments', 'Play with the Department table', 'SELECT t.ROWID, t.* FROM DEPT t'
, 'I,U,D', 'DEPT'); 


INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
2, 'DEPTNO', 'Num', NULL, 'Y', 'Y', 'M', NULL, 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
2, 'DNAME', 'Name', NULL, 'Y', 'Y', 'M', NULL, 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
2, 'LOC', 'Location', NULL, 'Y', 'Y', 'M', NULL, 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
1, 'EMPNO', 'EmpNo', NULL, 'Y', 'Y', 'U', NULL, 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
1, 'ENAME', 'Name', NULL, 'Y', 'Y', 'U', NULL, 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
1, 'JOB', 'Job', NULL, 'Y', 'Y', 'U', NULL, 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
1, 'MGR', 'Mgr', NULL, 'N', 'Y', 'U', NULL, 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
1, 'HIREDATE', 'HireDate', 'SYSDATE', 'Y', 'Y', 'U', 'DD/MM/YYYY', 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
1, 'SAL', 'Salary', '1000', 'N', 'Y', 'U', 'FM99999.00', 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
1, 'COMM', 'Comm.', NULL, 'N', 'Y', 'U', NULL, 'Y'); 
INSERT INTO DYN_COLS ( D_NUM, C_COL, C_LABEL, C_INITVALUE, C_REQUIRED, C_QUERYABLE, C_CASE, C_MASK,
C_VISIBLE ) VALUES ( 
1, 'DEPTNO', 'DeptNo', NULL, 'Y', 'Y', 'U', NULL, 'Y'); 


INSERT INTO DYN_VALIDATION ( D_NUM, V_COL, V_NUM, V_RULE, V_ERR_MSG ) VALUES ( 
1, 'SAL', 1, '<= 10000', 'Salary must be lower than 10000'); 
INSERT INTO DYN_VALIDATION ( D_NUM, V_COL, V_NUM, V_RULE, V_ERR_MSG ) VALUES ( 
1, 'HIREDATE', 1, '< ''30/12/2020''', 'Hiredate must be lower than 31/12/2020'); 
INSERT INTO DYN_VALIDATION ( D_NUM, V_COL, V_NUM, V_RULE, V_ERR_MSG ) VALUES ( 
1, 'DEPTNO', 1, 'EXISTS IN TABLE=DEPT COLUMN=DEPTNO', 'Department must exists in the DEPT table'); 
INSERT INTO DYN_VALIDATION ( D_NUM, V_COL, V_NUM, V_RULE, V_ERR_MSG ) VALUES ( 
1, 'EMPNO', 1, 'BETWEEN 1000 AND 9999', 'Empno must be between 1000 and 9999'); 

INSERT INTO DEPT ( DEPTNO, DNAME, LOC ) VALUES ( 
10, 'ACCOUNTING', 'NEW YORK'); 
INSERT INTO DEPT ( DEPTNO, DNAME, LOC ) VALUES ( 
20, 'RESEARCH', 'DALLAS'); 
INSERT INTO DEPT ( DEPTNO, DNAME, LOC ) VALUES ( 
30, 'SALES', 'CHICAGO'); 
INSERT INTO DEPT ( DEPTNO, DNAME, LOC ) VALUES ( 
40, 'OPERATIONS', 'BOSTON'); 

INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7369, 'SMITH', 'CLERK', 7902,  TO_Date( '12/17/1980 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 900, NULL, 20); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7499, 'ALLEN', 'SALESMAN', 7698,  TO_Date( '02/20/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 1600, 300, 30); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7521, 'WARD', 'SALESMAN', 7698,  TO_Date( '02/22/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 1250, 500, 30); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7566, 'JONES', 'MANAGER', 7839,  TO_Date( '04/02/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 2975, NULL, 20); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7654, 'MARTIN', 'SALESMAN', 7698,  TO_Date( '09/28/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 1250, 1400, 30); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7698, 'BLAKE', 'MANAGER', 7839,  TO_Date( '05/01/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 2850, NULL, 30); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7782, 'CLARK', 'MANAGER', 7839,  TO_Date( '06/09/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 2450, NULL, 10); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7788, 'SCOTT', 'ANALYST', 7566,  TO_Date( '12/09/1982 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 3000, NULL, 20); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7839, 'KING', 'PRESIDENT', NULL,  TO_Date( '12/17/1980 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 5000, NULL, 10); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7844, 'TURNER', 'SALESMAN', 7698,  TO_Date( '09/08/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 1500, 0, 30); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7876, 'ADAMS', 'CLERK', 7788,  TO_Date( '01/12/1983 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 1100, NULL, 20); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7900, 'JAMES', 'CLERK', 7698,  TO_Date( '12/03/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 950, NULL, 30); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7902, 'FORD', 'ANALYST', 7566,  TO_Date( '12/03/1981 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 3000, NULL, 20); 
INSERT INTO EMP ( EMPNO, ENAME, JOB, MGR, HIREDATE, SAL, COMM, DEPTNO ) VALUES ( 
7934, 'MILLER', 'CLERK', 7782,  TO_Date( '12/09/1982 12:00:00 AM', 'MM/DD/YYYY HH:MI:SS AM') , 1301, NULL, 10); 

COMMIT;

@@PKG_DYNAMIC.PKS
@@PKG_DYNAMIC_2.PKS
@@PKG_DYNAMIC.PKB
@@PKG_DYNAMIC_2.PKB
